<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
    xmlns:fo="http://www.w3.org/1999/XSL/Format">
    <xsl:output  method="xml" />
    <xsl:template match="/">
        <fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">
            <fo:layout-master-set>
                <fo:simple-page-master page-height="4.82in" page-width="3.57in"
                    master-name="default">
                    <fo:region-body/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            <fo:declarations>
                <x:xmpmeta xmlns:x="adobe:ns:meta/">
                    <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
                        <rdf:Description rdf:about="" xmlns:dc="http://purl.org/dc/elements/1.1/">
                            <dc:title>Free Software, Free Society: Selected Essays of Richard M. Stallman</dc:title>
                            <dc:creator>Richard M. Stallman</dc:creator>
                        </rdf:Description>
                    </rdf:RDF>
                </x:xmpmeta>
            </fo:declarations>
            <fo:bookmark-tree>
                <xsl:apply-templates select="/texinfo/node[nodename='Top']/menu/menuentry"
                    mode="bookmark"/>
            </fo:bookmark-tree>
            <fo:page-sequence master-reference="default">
                <fo:static-content flow-name="xsl-footnote-separator">
                    <fo:block text-align="center">
                        <fo:leader leader-pattern="rule" leader-length="80%" rule-style="solid"
                            rule-thickness="0.5pt"/>
                    </fo:block>
                </fo:static-content>
                <fo:flow font-size="10pt" flow-name="xsl-region-body">
                    <xsl:apply-templates select="/texinfo/titlepage/*" mode="title" />
                    <xsl:apply-templates select="/texinfo/node/unnumberedsec | /texinfo/node/unnumbered | /texinfo/node/chapter"/>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>
    </xsl:template>
    
    <xsl:template match="*" >
        <xsl:copy-of select="."/>
    </xsl:template>
    
    <xsl:template match="center">
        <fo:block text-align="center" >
            <xsl:apply-templates select="*|text()"></xsl:apply-templates>
        </fo:block>
    </xsl:template>
    
    <xsl:template match="image">
        <fo:block keep-with-previous.within-column="always" margin-top="2em" margin-bottom="2em">
        <fo:external-graphic width="100%" content-width="scale-to-fit" scaling="uniform"
        content-height="100%" src="url('{concat(@name,&quot;.gif&quot;)}')"/> 
        </fo:block>
    </xsl:template>

    
    <xsl:template match="center" mode="title">
        <fo:block text-align="center" >
            <xsl:apply-templates select="*|text()"></xsl:apply-templates>
        </fo:block>
    </xsl:template>
    
    <xsl:template match="image" mode="title">
        <fo:block keep-with-previous.within-column="always" margin-top="2em" margin-bottom="2em">
        <fo:external-graphic width="100%" content-width="scale-to-fit" scaling="uniform"
        content-height="100%" src="url('{concat(@name,&quot;.gif&quot;)}')"/> 
        </fo:block>
    </xsl:template>

    
    <xsl:template match="emph|dfn|cite|var">
        <fo:inline font-style="italic"><xsl:apply-templates select="*|text()" /></fo:inline>
    </xsl:template>
    
    <xsl:template match="code|uref|urefurl|samp">
        <fo:inline font-family="courier"><xsl:apply-templates select="*|text()" /></fo:inline>
    </xsl:template>
    
    <xsl:template match="group|sc">
      <xsl:apply-templates select="text()|*"/>
    </xsl:template>
    
    <xsl:template match="smallexample|display">
        <fo:block margin-bottom="2em" font-family="Courier" font-size="6" white-space="pre">
            <xsl:apply-templates select="text()|*"/>
        </fo:block>                
    </xsl:template>
        
    <xsl:template match="acronym">
        <fo:inline font-variant="small-caps"><xsl:value-of select="acronymword"/></fo:inline>
    </xsl:template>
    
    <xsl:template match="strong">
        <fo:inline font-weight="bold"><xsl:apply-templates select="*|text()" /></fo:inline>
    </xsl:template>
    
    <xsl:template match="xref">
        <fo:basic-link font-style="italic" internal-destination="{xrefnodename}"><xsl:value-of select="/texinfo/node[nodename=current()/xrefnodename]/*/title"/></fo:basic-link>
    </xsl:template>
    
    <xsl:template match="item">
        <xsl:param name="first"/>
        <xsl:param name="itemfunction" select="'&#x02022;'"/>        
        <fo:list-item>
            <fo:list-item-label end-indent="label-end()">
                <fo:block>
                    <xsl:choose>
                        <xsl:when test="$first=0"><xsl:value-of select="position()-1"/></xsl:when>
                        <xsl:when test="$first=1"><xsl:value-of select="position()"/></xsl:when>
                        <xsl:when test="$first='A'"><xsl:number format="A" /></xsl:when>
                        <xsl:otherwise><xsl:value-of select="$itemfunction"/></xsl:otherwise>
                    </xsl:choose>
                </fo:block>
            </fo:list-item-label>
            <fo:list-item-body start-indent="body-start()">
                <xsl:choose>
                    <xsl:when test="normalize-space(./text()) != ''">
                        <fo:block>
                            <xsl:apply-templates select="*|text()" />            
                        </fo:block>
                    </xsl:when>
                    <xsl:otherwise><xsl:apply-templates select="*|text()" /></xsl:otherwise>
                </xsl:choose>
            </fo:list-item-body>
        </fo:list-item>        
    </xsl:template>
    
    <xsl:template match="itemize">
        <fo:list-block provisional-distance-between-starts="1.5em"
            provisional-label-separation="1em">
            <xsl:apply-templates select="item">
                <xsl:with-param name="itemfunction" select="itemfunction"/>
            </xsl:apply-templates>
        </fo:list-block>
    </xsl:template>
    
    <xsl:template match="enumerate">
        <fo:list-block provisional-distance-between-starts="1.5em"
            provisional-label-separation="1em">
            <xsl:apply-templates select="item" >
                <xsl:with-param name="first" select="@first" />
            </xsl:apply-templates>
        </fo:list-block>
    </xsl:template>
        
    <xsl:template match="tableitem" mode="termtable">
        <fo:block keep-with-next.within-column="always" font-weight="bold"><xsl:apply-templates select="tableterm/text()"/></fo:block>
        <fo:block margin-left="2em">
        <xsl:apply-templates select="item/*|item/text()"/>
        </fo:block>
    </xsl:template>
    
    <xsl:template match="table">
        <xsl:apply-templates select="tableitem" mode="termtable" />
    </xsl:template>
           
    <xsl:template match="logo">
        <!-- FIXME: Use correct TeX logo -->
        <xsl:value-of select="text()"/>
    </xsl:template>
    
    <xsl:template match="title">
        <fo:block margin-bottom="1em" margin-top="1em" keep-with-next.within-column="always" font-weight="bold">
            <xsl:value-of select="text()"/>
        </fo:block>        
    </xsl:template>
    
    <xsl:template match="title[parent::unnumbered or parent::chapter]">
        <fo:block font-size="12" break-before="page" margin-top="1em" margin-bottom="1em" keep-with-next.within-column="always" font-weight="bold">
            <xsl:attribute name="id"><xsl:value-of select="../../nodename"/></xsl:attribute>
            <xsl:value-of select="text()"/>
        </fo:block>        
    </xsl:template>
    
    <xsl:template match="quotation">
        <fo:block margin-left="2em" margin-right="2em" font-style="italic">
            <xsl:apply-templates select="*|text()" />
        </fo:block>
    </xsl:template>

    <xsl:template match="titlefont" mode="title">
      <fo:block text-align="center">
        <fo:inline font-weight="bold"><xsl:apply-templates select="*|text()" /></fo:inline> 
      </fo:block>
    </xsl:template> 

    <xsl:template match="para" mode="title">
      <fo:block>
        <xsl:if test="preceding-sibling::*[1][name() = 'sp']">
          <xsl:attribute name="margin-top"><xsl:value-of select="preceding-sibling::sp/@lines"/>em</xsl:attribute>
        </xsl:if>
        <xsl:apply-templates select="*|text()" />
      </fo:block>
    </xsl:template>

    <xsl:template match="row">
      <fo:table-row>
        <xsl:apply-templates select="entry" />
      </fo:table-row>
    </xsl:template>

    <xsl:template match="entry">
      <fo:table-cell>
        <fo:block margin-top="0.5em" margin-bottom="0.5em">
          <xsl:value-of select="text()" />
        </fo:block>
      </fo:table-cell>
    </xsl:template>

    <xsl:template match="multitable" mode="title">
      <fo:table>
        <fo:table-body>
          <xsl:apply-templates select="tbody/row" />
        </fo:table-body>
      </fo:table>
    </xsl:template>
    
    <xsl:template match="titlefont">
        <fo:inline font-weight="bold"><xsl:apply-templates select="*|text()" /></fo:inline> 
    </xsl:template>
    
    <xsl:template match="heading">
        <fo:block font-weight="bold" keep-with-next.within-column="always" margin-top="0.5em" margin-bottom="0.5em"><xsl:apply-templates select="*|text()" /></fo:block>
    </xsl:template>
             
    <xsl:template match="unnumbered|chapter|unnumberedsec">
        <xsl:apply-templates select="*" />        
    </xsl:template>
    
    <xsl:template match="para">
        <xsl:param name="linked" select="false()" />
        <fo:block text-align="justify" margin-bottom="0.5em">            
            <xsl:if test="$linked">* </xsl:if>
        <xsl:apply-templates select="*|text()"></xsl:apply-templates>
        </fo:block>
    </xsl:template>
    
    <xsl:template match="footnote">
        <fo:footnote><fo:inline font-size="7">* </fo:inline><fo:footnote-body font-size="8pt"><xsl:apply-templates select="*|text()"><xsl:with-param name="linked" select="true()" /></xsl:apply-templates></fo:footnote-body></fo:footnote>
    </xsl:template>
    
    <xsl:template match="para[child::footnote and normalize-space(./text()) = '']">
        <fo:block>
        <fo:footnote><fo:inline/><fo:footnote-body font-size="8pt"><xsl:apply-templates select="footnote/*" /></fo:footnote-body></fo:footnote>
        </fo:block>
    </xsl:template>
        
    <xsl:template match="punct[@end-of-sentence]|sp|indexterm"></xsl:template>    
    
    <xsl:template match="menuentry" mode="bookmark">
        <fo:bookmark internal-destination="{menunode}">
            <fo:bookmark-title>
                <xsl:value-of select="/texinfo/node[nodename=current()/menunode]/*/title"/>
            </fo:bookmark-title>
        </fo:bookmark>
    </xsl:template>
</xsl:stylesheet>
